from PyQt6.QtGui import QSyntaxHighlighter, QTextCharFormat, QColor
from PyQt6.QtCore import QRegularExpression


class PythonHighlighter(QSyntaxHighlighter):
    """A syntax highlighter for Python code."""

    def highlightBlock(self, text: str):
        self.light(text)
        self.highlight_numbers(text)
        self.highlight_keywords(text)
        self.highlight_strings(text)
        self.highlight_comments(text)

    def light(self, text: str):
        light_format = QTextCharFormat()
        light_format.setForeground(QColor(214, 214, 214))
        light_format.setFontFamily("Consolas")
        light_format.setFontPointSize(16)
        light_format.setFontWeight(150)
        self.highlight(QRegularExpression(r"."), light_format, text)
    
    def highlight_numbers(self, text: str):
        number_format = QTextCharFormat()
        number_format.setForeground(QColor(180, 210, 115))
        number_format.setFontFamily("Consolas")
        number_format.setFontPointSize(16)
        number_format.setFontWeight(150)
        self.highlight(QRegularExpression(r"\b[0-9]+\b"), number_format, text)

    def highlight_keywords(self, text: str):
        keyword_format = QTextCharFormat()
        keyword_format.setForeground(QColor(176, 82, 121))
        keyword_format.setFontFamily("Consolas")
        keyword_format.setFontPointSize(16)
        keyword_format.setFontWeight(150)
        self.highlight(QRegularExpression(
            r"\b("
            r"and|as|assert|async|await"
            r"|break|class|continue|def|"
            r"del|elif|else|except|finally|"
            r"for|from|global|if|import|in|"
            r"is|lambda|nonlocal|not|or"
            r"|pass|raise|return|try|while"
            r"|with|yield|self|super|True|"
            r"False|None|print|input|open|"
            r"range|len|list|dict|set|tuple|"
            r"int|float|str|bool|complex|"
            r"abs|all|any|ascii|bin|bool|"
            r"bytes|callable|chr|classmethod|"
            r"compile|complex|delattr|dict|"
            r"dir|divmod|enumerate|eval|"
            r"exec|filter|float|format|frozenset|"
            r"getattr|globals|hasattr|hash|"
            r"help|hex|id|input|int|isinstance|"
            r"issubclass|iter|len|list|locals|"
            r"map|max|memoryview|min|next|"
            r"object|oct|open|ord|pow|print|"
            r"property|range|repr|reversed|"
            r"round|set|setattr|slice|sorted|"
            r"staticmethod|str|sum|super|tuple|"
            r"type|vars|zip"
            r")\b"
        ), keyword_format, text)

    def highlight_strings(self, text: str):
        string_format = QTextCharFormat()
        string_format.setForeground(QColor(206, 145, 120))
        string_format.setFontFamily("Consolas")
        string_format.setFontPointSize(16)
        string_format.setFontWeight(150)
        self.highlight(QRegularExpression(r"\".*\""), string_format, text)
        self.highlight(QRegularExpression(r"'.*'"), string_format, text)

    def highlight_comments(self, text: str):
        comment_format = QTextCharFormat()
        comment_format.setForeground(QColor(100, 100, 100))
        comment_format.setFontFamily("Consolas")
        comment_format.setFontPointSize(16)
        comment_format.setFontWeight(150)
        self.highlight(QRegularExpression(r"#.*"), comment_format, text)

    def highlight(self, pattern: QRegularExpression, char_format: QTextCharFormat, text: str):
        match = pattern.match(text)
        while match.hasMatch():
            start = match.capturedStart()
            length = match.capturedLength()
            self.setFormat(start, length, char_format)
            match = pattern.match(text, start + length)



from copy import copy
from pathlib import Path
from subprocess import Popen
from typing import Union
from types import NoneType
import shutil
import string

from PyQt6.QtWidgets import (
    QVBoxLayout,
    QHBoxLayout,
    QMainWindow,
    QWidget,
    QColorDialog,
    QTextEdit,
    QPushButton,
    QFileDialog,
    QInputDialog,
    QMessageBox,
    QComboBox,
    QMenu
)
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QPixmap, QIcon, QAction, QColor

from manim import (
    Scene,
    Camera,
    Circle,
    RegularPolygon,
    Line,
    MathTex,
    Text,
    VMobject,
    rgb_to_color,
    BLACK,
    DL,
    UR
)

from PIL.ImageQt import ImageQt
from PIL import Image

from .api import ManimStudioAPI
from .mobject_picker import MobjectPicker
from .preview import Preview
from .syntax_highlighting import PythonHighlighter


FILE_CONTENT_TEMPLATE = """
from manim import *
%s

class %s(%s):
    def construct(self):
        super().construct()
%s
""".strip()


CONTENT_IF_IMPORTED = """
from manim_studio.utils import import_from_file

from pathlib import Path


module = import_from_file(Path(%s))
""".lstrip()


class Window(QMainWindow):
    """A window to preview the scene."""

    def __init__(
        self,
        window_size: tuple[int, int],
        scene: Scene
    ):
        super().__init__(None)
        self.setWindowTitle("Manim Studio")
        self.saved_file = None
        self.saved_scene_class_name = None
        self.scene = scene
        self.setMouseTracking(True)

        self.setCentralWidget(QWidget())
        layout = QHBoxLayout()
        self.centralWidget().setLayout(layout)
        v_layout_1 = QVBoxLayout()
        layout.addLayout(v_layout_1)
        v_layout_2 = QVBoxLayout()
        layout.addLayout(v_layout_2)

        edit_menu = QMenu("Edit", self)
        set_fill_color_action = QAction("Set Fill Color", self)
        set_fill_color_action.triggered.connect(self.set_fill_color)
        edit_menu.addAction(set_fill_color_action)
        set_stroke_color_action = QAction("Set Stroke Color", self)
        set_stroke_color_action.triggered.connect(self.set_stroke_color)
        edit_menu.addAction(set_stroke_color_action)
        set_stroke_width_action = QAction("Set Stroke Width", self)
        set_stroke_width_action.triggered.connect(self.set_stroke_width)
        edit_menu.addAction(set_stroke_width_action)
        self.menuBar().addMenu(edit_menu)
        self.menuBar().setNativeMenuBar(False)

        self.generate_python_file_button = QPushButton("Generate Python File")
        self.generate_python_file_button.clicked.connect(self.generate_python_file)
        v_layout_2.addWidget(self.generate_python_file_button)
        self.generate_video_file_button = QPushButton("Render Video File")
        self.generate_video_file_button.clicked.connect(self.render_video_file)
        v_layout_2.addWidget(self.generate_video_file_button)

        self.mobject_picker_combobox = QComboBox()
        self.setup_mobject_picker_combobox()
        v_layout_2.addWidget(self.mobject_picker_combobox)

        self.add_mobject_button = QPushButton("Add Mobject")
        self.add_mobject_button.setShortcut("Ctrl+A")
        self.add_mobject_button.clicked.connect(self.select_mobject)
        v_layout_2.addWidget(self.add_mobject_button)

        self.label = Preview(window_size, self.mobject_picker_combobox, scene)
        v_layout_1.addWidget(self.label)

        self.editor = QTextEdit()
        self.editor.setPlaceholderText("Write your code here")
        self.highlighter = PythonHighlighter(self.editor.document())
        self.editor.setFixedHeight(window_size[1] // 2 - 100)
        self.editor.setFixedWidth(window_size[0] // 2)
        self.editor.setContentsMargins(0, 0, 0, 0)
        self.editor.setStyleSheet("background-color: rgb(46, 46, 46);")
        v_layout_1.addWidget(self.editor)

        old_update_to_time = copy(scene.update_to_time)

        def update_to_time(time: float):
            old_update_to_time(time)
            self.update_image()
        
        scene.update_to_time = update_to_time

        self.execute_button = QPushButton("Execute")
        self.execute_button.clicked.connect(lambda: self.execute(self.editor.toPlainText()))
        self.execute_button.setShortcut("Ctrl+Return")
        v_layout_1.addWidget(self.execute_button)
    
    def execute(self, code: str):
        setattr(self.scene, "code", code)
    
    def set_fill_color(self):
        mobject_picker = MobjectPicker(
            self.width(),
            self.scene.camera.frame_width / self.scene.camera.frame_height,
            VMobject
        )
        selected_mobject = mobject_picker.wait_for_selection()
        if selected_mobject is None:
            return
        name, selected_mobject = selected_mobject
        selected_mobject: VMobject
        color = selected_mobject.get_fill_color()
        (r, g, b), a = color.to_rgb(), selected_mobject.get_fill_opacity()
        color = int(r*255), int(g*255), int(b*255), int(a*255)
        fill_color = QColorDialog.getColor(QColor(*color), self, "Select Fill Color", QColorDialog.ColorDialogOption.ShowAlphaChannel)
        if not fill_color.isValid():
            return
        r, g, b, a = fill_color.getRgbF()
        ManimStudioAPI.execute(f"{name}.set_fill(rgb_to_color(({r}, {g}, {b})), {a})")
        QMessageBox.information(self, "Fill Color Set", "The fill color has been set successfully")
    
    def set_stroke_color(self):
        mobject_picker = MobjectPicker(
            self.width(),
            self.scene.camera.frame_width / self.scene.camera.frame_height,
            VMobject
        )
        selected_mobject = mobject_picker.wait_for_selection()
        if selected_mobject is None:
            return
        name, selected_mobject = selected_mobject
        selected_mobject: VMobject
        color = selected_mobject.get_stroke_color()
        (r, g, b), a = color.to_rgb(), selected_mobject.get_stroke_opacity()
        color = int(r*255), int(g*255), int(b*255), int(a*255)
        stroke_color = QColorDialog.getColor(QColor(*color), self, "Select Stroke Color", QColorDialog.ColorDialogOption.ShowAlphaChannel)
        if not stroke_color.isValid():
            return
        r, g, b, a = stroke_color.getRgbF()
        ManimStudioAPI.execute(f"{name}.set_stroke(rgb_to_color(({r}, {g}, {b})), opacity={a})")
        QMessageBox.information(self, "Stroke Color Set", "The stroke color has been set successfully")
    
    def set_stroke_width(self):
        mobject_picker = MobjectPicker(
            self.width(),
            self.scene.camera.frame_width / self.scene.camera.frame_height,
            VMobject
        )
        selected_mobject = mobject_picker.wait_for_selection()
        if selected_mobject is None:
            return
        name, selected_mobject = selected_mobject
        selected_mobject: VMobject
        stroke_width, ok = QInputDialog.getDouble(self, "Set Stroke Width", "Enter the stroke width", selected_mobject.get_stroke_width(), 0, 100, 1)
        if not ok:
            return
        ManimStudioAPI.execute(f"{name}.set_stroke(width={stroke_width})")
        QMessageBox.information(self, "Stroke Width Set", "The stroke width has been set successfully")
    
    def select_mobject(self):
        self.label.mobject_to_put = ManimStudioAPI.supported_mobjects[self.mobject_picker_combobox.currentText()]
        self.mobject_picker_combobox.setDisabled(True)
    
    def setup_mobject_picker_combobox(self):
        for mobject in ManimStudioAPI.supported_mobjects:
            if mobject == "Circle":
                camera = Camera(
                    pixel_width=1000,
                    pixel_height=1000,
                    frame_width=2.25,
                    frame_height=2.25,
                    background_opacity=0
                )
                self.mobject_picker_combobox.addItem(
                    QIcon(QPixmap.fromImage(ImageQt(Circle(stroke_width=16) \
                                                    .get_image(camera)))),
                    "Circle"
                )
            elif mobject == "Regular Polygon":
                camera = Camera(
                    pixel_width=1000,
                    pixel_height=1000,
                    frame_width=2.25,
                    frame_height=2.25,
                    background_opacity=0
                )
                self.mobject_picker_combobox.addItem(
                    QIcon(QPixmap.fromImage(ImageQt(RegularPolygon(6, stroke_width=16) \
                                                    .get_image(camera)))),
                    "Regular Polygon"
                )
            elif mobject == "Line":
                camera = Camera(
                    pixel_width=1000,
                    pixel_height=1000,
                    frame_width=2.25,
                    frame_height=2.25,
                    background_opacity=0
                )
                self.mobject_picker_combobox.addItem(
                    QIcon(QPixmap.fromImage(ImageQt(Line(
                        DL, UR, stroke_width=16, color=BLACK
                    ).get_image(camera)))),
                    "Line"
                )
            elif mobject == "MathTex":
                camera = Camera(
                    pixel_width=1000,
                    pixel_height=1000,
                    frame_width=2.25,
                    frame_height=2.25,
                    background_opacity=0
                )
                self.mobject_picker_combobox.addItem(
                    QIcon(QPixmap.fromImage(ImageQt(MathTex("x^2", color=BLACK) \
                                                    .scale_to_fit_width(2) \
                                                    .get_image(camera)))),
                    "MathTex"
                )
            elif mobject == "Text":
                camera = Camera(
                    pixel_width=1000,
                    pixel_height=1000,
                    frame_width=2.25,
                    frame_height=2.25,
                    background_opacity=0
                )
                self.mobject_picker_combobox.addItem(
                    QIcon(QPixmap.fromImage(ImageQt(Text("T", color=BLACK) \
                                                    .scale_to_fit_width(2) \
                                                    .get_image(camera)))),
                    "Text"
                )
    
    @staticmethod
    def make_camel_case(text: str, default: str) -> str:
        text = text.lstrip(string.punctuation + string.whitespace + string.digits)
        text = text.rstrip(string.punctuation + string.whitespace)
        if text == "":
            return default
        text = text.split()
        text = "".join(word.capitalize() for word in text)
        return text

    def generate_python_file(self):
        file_name, _ = QFileDialog.getSaveFileName(self, "Save Python File", "", "Python Files (*.py)")
        if not file_name:
            return
        scene_class_name = QInputDialog.getText(self, "Scene Class Name", "Enter the name of the scene class")[0]
        scene_class_name = self.make_camel_case(
            scene_class_name,
            f"{ManimStudioAPI.scene.__class__.__name__}Generated"
        )
        codes = ManimStudioAPI.codes
        codes = [line for code in codes for line in code.split("\n")]
        with open(file_name, "w") as f:
            f.write(FILE_CONTENT_TEMPLATE % (
                CONTENT_IF_IMPORTED % repr(str(Path(ManimStudioAPI.path_to_file).absolute()))
                if ManimStudioAPI.path_to_file else "",
                scene_class_name,
                f"module.{ManimStudioAPI.scene.__class__.__name__}"
                if ManimStudioAPI.scene.__class__ != Scene else "Scene",
                "\n".join(8*" " + line for line in codes)
            ))
        self.saved_file = file_name
        self.saved_scene_class_name = scene_class_name
        QMessageBox.information(self, "File Saved", "The file has been saved successfully")
    
    def render_video_file(self):
        if not self.saved_file:
            QMessageBox.critical(self, "Error", "You must generate a Python file before rendering")
            return
        file_name, _ = QFileDialog.getSaveFileName(self, "Save Video File", "", "Video Files (*.mp4)")
        if not file_name:
            return
        preview = QMessageBox.question(self, "Preview", "Do you want to preview the video?", QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No)
        path = Path(file_name)

        process = Popen(
            [
                "manim",
                self.saved_file,
                self.saved_scene_class_name,
                "-o",
                f"{path.stem}.mp4",
                "--format=mp4",
            ] + (["-p"] if preview == QMessageBox.StandardButton.Yes else [])
        )
        process.wait()

        output_path = Path("media") / "videos" / Path(self.saved_file).stem / "1080p60" / f"{path.stem}.mp4"
        if output_path.exists():
            shutil.move(str(output_path), str(path))
            QMessageBox.information(self, "File Saved", "The video has been rendered successfully")
        else:
            QMessageBox.critical(self, "Error", "The video could not be rendered")
    
    def update_image(self):
        frame = self.scene.renderer.get_frame()
        qimage = ImageQt(Image.fromarray(frame))
        pixmap = QPixmap.fromImage(qimage) \
            .scaled(self.label.width(), self.label.height(), Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.SmoothTransformation)
        self.label.setPixmap(pixmap)



from typing import Union
from types import NoneType

from PyQt6.QtWidgets import QDialog, QGridLayout, QPushButton
from PyQt6.QtGui import QIcon, QPixmap
from PyQt6.QtCore import QSize

from manim import Mobject

from PIL.ImageQt import ImageQt

from .api import ManimStudioAPI


class MobjectPicker(QDialog):
    def __init__(self, window_width: int, aspect_ratio: float, mobject_class: type[Mobject]):
        super().__init__()
        self.setLayout(QGridLayout())
        self.selected_mobject = None
        self.setWindowTitle("Mobject Picker")
        
        self.mobjects_available = {k: v for k, v in ManimStudioAPI.scope.items() if isinstance(v, mobject_class)}
        self.mobjects_buttons = {}

        for i, (name, mobject) in enumerate(self.mobjects_available.items()):
            icon = QIcon(QPixmap.fromImage(ImageQt(mobject.get_image())))
            button = QPushButton(icon, None)
            button.setIconSize(QSize(window_width // 3, int(window_width // 3 / aspect_ratio)))
            button.setFixedWidth(window_width // 3)
            button.setFixedHeight(int(button.width() / aspect_ratio))
            self.layout().addWidget(button, i // 3, i % 3)
            self.mobjects_buttons[button] = mobject
            button.clicked.connect(lambda: self.select_mobject(name))

    def select_mobject(self, name: str):
        self.selected_mobject = name, self.mobjects_available[name]
        self.close()
    
    def wait_for_selection(self) -> Union[NoneType, tuple[str, Mobject]]:
        self.exec()
        return self.selected_mobject



import importlib
from types import ModuleType
from pathlib import Path

from manim import Scene


def import_from_file(path_to_file: Path) -> ModuleType:
    """Import a module from a file."""
    spec = importlib.util.spec_from_file_location(path_to_file.stem, path_to_file)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module


def qt_coords_to_manim_coords(
    scene: Scene, 
    x: float,
    y: float,
    label_x: int,
    label_y: int,
    label_width: int,
    label_height: int
) -> tuple[float, float]:
    """Convert pixel coordinates to Manim coordinates."""
    x -= label_x
    y -= label_y
    x /= label_width
    y /= label_height
    x *= scene.camera.frame_width
    y *= scene.camera.frame_height
    x -= scene.camera.frame_width / 2
    y -= scene.camera.frame_height / 2
    y *= -1
    return x, y
    



from typing import Union
from types import ModuleType, NoneType
import time

import manim

from PyQt6.QtWidgets import QMessageBox


def hold_on(scene: manim.Scene):
    """Hold the scene until the user executes something on Manim Studio."""
    if not ManimStudioAPI.enabled:
        return
    scene.code = None
    last_time = time.time()
    while scene.code is None:
        dt = time.time() - last_time
        scene.wait(dt, frozen_frame=False)
        last_time = time.time()
    ManimStudioAPI.execute(scene.code)
    hold_on(scene)


class ManimStudioAPI:
    """The API for Manim Studio"""
    enabled: bool = False
    supported_mobjects: dict[str, type[manim.Mobject]] = {
        "Circle": manim.Circle,
        "Regular Polygon": manim.RegularPolygon,
        "Line": manim.Line,
        "MathTex": manim.MathTex,
        "Text": manim.Text
    }

    def __new__(
        cls,
        scene: manim.Scene,
        module: Union[ModuleType, NoneType],
        path_to_file: Union[str, NoneType],
        plugins: list[ModuleType]
    ):
        if not cls.enabled:
            return

        cls.scene = scene
        cls.scope = globals().copy()
        cls.scope["self"] = scene
        cls.update_scope_with_module(manim)
        cls.path_to_file = path_to_file

        if type(scene) == manim.Scene:
            def new_construct():
                hold_on(scene)
            scene.construct = new_construct

        if module is not None:
            cls.update_scope_with_module(module)

        for plugin in plugins:
            cls.add_plugin(plugin)
        
        cls.codes = []

        return super().__new__(cls)
    
    @classmethod
    def execute(cls, code: str):
        """Execute the given code in the scope of the scene.
        
        Warning
        -------
        The code is executed directly. This means that it can be dangerous to
        use this method with untrusted code. Use it at your own risk.
        """
        state = cls.scene.__dict__.copy()
        try:
            exec(code, cls.scope)
        except Exception as e:
            cls.scene.__dict__ = state
            cls.show_error(e)
        else:
            cls.codes.append(code)
    
    @classmethod
    def show_error(cls, error: Exception):
        """Show the given error in the GUI."""
        QMessageBox.critical(
            None,
            "Error",
            "¡Ha ocurrido un error!\n\n"
            f"{error.__class__.__name__}: {error}"
        )
    
    @classmethod
    def update_scope_with_module(cls, module: ModuleType):
        """Update the scope of the scene with the given module.
        It is useful to add new variables or functions that you want to use.
        """
        cls.scope.update(module.__dict__)
    
    @classmethod
    def add_plugin(
        cls,
        plugin: ModuleType
    ):
        """Add a plugin to Manim Studio application.
        
        Parameters
        ----------
        plugin : ModuleType
            The module of the plugin to add. It must contain a :code:`main` function
            that will be called with the API as the only argument."""
        if not hasattr(plugin, "main"):
            raise ValueError("The plugin must have a 'main' function")
        if not callable(plugin.main):
            raise ValueError("The 'main' function of the plugin must be callable")
        if plugin.main.__code__.co_argcount != 1:
            raise ValueError("The 'main' function of the plugin must take exactly one argument")
        cls.plugins[plugin.__name__] = plugin
    
    @classmethod
    def run_plugin(
        cls,
        plugin_name: str
    ):
        """Run the plugin with the given name."""
        if plugin_name not in cls.plugins:
            raise ValueError(f"The plugin '{plugin_name}' does not exist")
        cls.plugins[plugin_name].main(cls)
    
    @classmethod
    def get_all_code(cls):
        """Return all the code executed in the application."""
        return "\n".join(cls.codes)



import string

from PyQt6.QtWidgets import QLabel, QComboBox, QInputDialog, QMessageBox
from PyQt6.QtGui import QMouseEvent

from manim import (
    Scene,
    Circle,
    RegularPolygon,
    Line,
    MathTex,
    Text,
    Mobject,
    DL,
    UR
)

import numpy as np

from .api import ManimStudioAPI
from .utils import qt_coords_to_manim_coords


class Preview(QLabel):
    def __init__(
        self,
        window_size: tuple[int, int],
        mobject_picker_combobox: QComboBox,
        scene: Scene
    ):
        super().__init__()
        self.saved_state = None
        self.setting_radius = None
        self.setting_line_start = None
        self.setting_line_end = None
        self.mobject_to_put = None
        self.setting_mathtex_size = None
        self.mathtex_string = None
        self.setting_text_size = None
        self.text_string = None
        self.scene = scene
        self.mobject_picker_combobox = mobject_picker_combobox
        self.setMouseTracking(True)

        window_width, window_height = window_size
        max_width = window_width // 2
        max_height = window_height // 2
        pixel_width, pixel_height = scene.camera.pixel_width, scene.camera.pixel_height
        aspect_ratio = pixel_width / pixel_height

        if aspect_ratio > max_width / max_height:
            width = max_width
            height = max_width // aspect_ratio
        else:
            width = int(max_height * aspect_ratio)
            height = max_height

        self.setFixedSize(width, height)
    
    @staticmethod
    def make_snake_case(name: str) -> str:
        name = name.lstrip(string.digits + string.punctuation + string.whitespace)
        name = name.rstrip(string.punctuation + string.whitespace)
        name = name.split()
        name = "_".join(name)
        name = name.lower()
        return name
    
    def add_mobject(self, mobject: Mobject):
        if isinstance(mobject, Circle):
            self.setting_radius = mobject
            return
        if isinstance(mobject, RegularPolygon):
            number_of_sides = QInputDialog.getInt(self, "Set number of sides", "Enter the number of sides (default: 6)")[0]
            number_of_sides = number_of_sides or 6
            side_length = QInputDialog.getDouble(self, "Set side length", "Enter the side length (default: 1)")[0]
            side_length = side_length or 1
            # Using radius formula given the side length and number of sides
            radius = side_length / (2 * np.sin(np.pi / number_of_sides))
            mobject = RegularPolygon(number_of_sides, radius=radius).move_to(mobject.get_center())
            name = QInputDialog.getText(self, "Set name", "Enter the mobject's name (default: reg_poly)")[0]
            name = self.make_snake_case(name) or "reg_poly"
            self.scene.__dict__ = self.saved_state
            self.saved_state = None
            ManimStudioAPI.execute(f"""
{name} = RegularPolygon({number_of_sides}, radius={radius}).move_to(RIGHT*{mobject.get_x()} + UP*{mobject.get_y()})
self.add({name})
""".strip())
            self.mobject_picker_combobox.setDisabled(False)
            return
        if isinstance(mobject, Line):
            self.setting_line_start = mobject
            return
        if isinstance(mobject, MathTex):
            center = mobject.get_center()
            error = True
            while error:
                try:
                    text = QInputDialog.getText(self, "Set text", "Enter the text (default: x^2)")[0]
                    text = text or "x^2"
                    mobject.become(MathTex(text).move_to(center))
                except (ValueError, RuntimeError):
                    QMessageBox.critical(self, "Error", "Invalid LaTeX code, try again")
                else:
                    error = False
            QMessageBox.information(self, "Info", "LaTeX rendered successfully")
            self.setting_mathtex_size = mobject
            self.mathtex_string = text
            return
        if isinstance(mobject, Text):
            center = mobject.get_center()
            text = QInputDialog.getText(self, "Set text", "Enter the text (default: T)")[0]
            text = text or "T"
            mobject.become(Text(text).move_to(center))
            self.setting_text_size = mobject
            self.text_string = text
            return
    
    def mouseMoveEvent(self, a0: QMouseEvent) -> None:
        super().mouseMoveEvent(a0)
        if (
            not self.mobject_to_put
            and not self.setting_radius
            and not self.setting_line_start
            and not self.setting_line_end
            and not self.setting_mathtex_size
            and not self.setting_text_size
        ):
            return
        qt_coords = a0.globalPosition()
        frame_x, frame_y = qt_coords_to_manim_coords(
            self.scene,
            qt_coords.x(),
            qt_coords.y(),
            self.x(),
            self.y(),
            self.width(),
            self.height()
        )
        frame_pos = np.array([frame_x, frame_y, 0])
        if self.setting_radius:
            center = self.setting_radius.get_center()
            radius = np.linalg.norm(frame_pos - center)
            self.setting_radius.become(Circle(radius=radius).move_to(center))
            self.setting_radius.radius = radius
            return
        if self.setting_line_start:
            self.setting_line_start.put_start_and_end_on(frame_pos, self.setting_line_start.get_end())
            return
        if self.setting_line_end:
            self.setting_line_end.put_start_and_end_on(self.setting_line_end.get_start(), frame_pos)
            return
        if self.setting_mathtex_size:
            center = self.setting_mathtex_size.get_center()
            self.setting_mathtex_size.scale_to_fit_height(2 * abs(frame_y - self.setting_mathtex_size.get_y()))
            self.setting_mathtex_size.move_to(center)
            return
        if self.setting_text_size:
            center = self.setting_text_size.get_center()
            self.setting_text_size.scale_to_fit_height(2 * abs(frame_y - self.setting_text_size.get_y()))
            self.setting_text_size.move_to(center)
            return
        if isinstance(self.mobject_to_put, type):
            self.saved_state = self.scene.__dict__.copy()
            if self.mobject_to_put == Circle:
                self.mobject_to_put = Circle()
            elif self.mobject_to_put == RegularPolygon:
                self.mobject_to_put = RegularPolygon(6)
            elif self.mobject_to_put == Line:
                self.mobject_to_put = Line(DL, UR)
            elif self.mobject_to_put == MathTex:
                self.mobject_to_put = MathTex("x^2")
            elif self.mobject_to_put == Text:
                self.mobject_to_put = Text("T")
            self.scene.add(self.mobject_to_put)
        self.mobject_to_put.move_to(frame_pos)
    
    def mousePressEvent(self, a0: QMouseEvent) -> None:
        super().mousePressEvent(a0)
        if (
            not self.mobject_to_put
            and not self.setting_radius
            and not self.setting_line_start
            and not self.setting_line_end
            and not self.setting_mathtex_size
            and not self.setting_text_size
        ):
            return
        if self.setting_radius:
            circle = self.setting_radius
            self.setting_radius = None
            name = QInputDialog.getText(self, "Set name", "Enter the circle's name (default: circ)")[0]
            name = self.make_snake_case(name) or "circ"
            self.scene.__dict__ = self.saved_state
            self.saved_state = None
            ManimStudioAPI.execute(f"""
{name} = Circle(radius={circle.radius}).move_to(RIGHT*{circle.get_x()} + UP*{circle.get_y()})
self.add({name})
""".strip())
            self.mobject_picker_combobox.setDisabled(False)
            return
        if self.setting_line_start:
            self.setting_line_end = self.setting_line_start
            self.setting_line_start = None
            return
        if self.setting_line_end:
            line = self.setting_line_end
            self.setting_line_end = None
            name = QInputDialog.getText(self, "Set name", "Enter the line's name (default: line)")[0]
            name = self.make_snake_case(name) or "line"
            self.scene.__dict__ = self.saved_state
            self.saved_state = None
            start_x, start_y, _ = line.get_start()
            end_x, end_y, _ = line.get_end()
            ManimStudioAPI.execute(f"""
{name} = Line(RIGHT*{start_x} + UP*{start_y}, RIGHT*{end_x} + UP*{end_y})
self.add({name})
""".strip())
            self.mobject_picker_combobox.setDisabled(False)
            return
        if self.setting_mathtex_size:
            mathtex = self.setting_mathtex_size
            self.setting_mathtex_size = None
            text = QInputDialog.getText(self, "Set name", "Enter the MathTex's name (default: math_tex)")[0]
            text = self.make_snake_case(text) or "math_tex"
            self.scene.__dict__ = self.saved_state
            self.saved_state = None
            ManimStudioAPI.execute(f"""
{text} = MathTex({repr(self.mathtex_string)}).scale_to_fit_height({mathtex.height}).move_to(RIGHT*{mathtex.get_x()} + UP*{mathtex.get_y()})
self.add({text})
""".strip())
            self.mathtex_string = None
            self.mobject_picker_combobox.setDisabled(False)
            return
        if self.setting_text_size:
            text = self.setting_text_size
            self.setting_text_size = None
            name = QInputDialog.getText(self, "Set name", "Enter the Text's name (default: text)")[0]
            name = self.make_snake_case(name) or "text"
            self.scene.__dict__ = self.saved_state
            self.saved_state = None
            ManimStudioAPI.execute(f"""
{name} = Text({repr(self.text_string)}).scale_to_fit_height({text.height}).move_to(RIGHT*{text.get_x()} + UP*{text.get_y()})
self.add({name})
""".strip())
            self.text_string = None
            self.mobject_picker_combobox.setDisabled(False)
            return
        mobject = self.mobject_to_put
        self.mobject_to_put = None
        self.add_mobject(mobject)



import argparse
from pathlib import Path
from types import ModuleType
from threading import Thread

from PyQt6.QtWidgets import QApplication

from manim import Scene
from manim._config import config

from .api import ManimStudioAPI
from .utils import import_from_file
from .window import Window


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--file", help="The file to open")
    parser.add_argument("--scene", help="The scene to open")
    parser.add_argument("--plugins", nargs="*", help="The plugins to load")
    args = parser.parse_args()

    if args.scene and not args.file:
        raise ValueError("You must provide a file when providing a scene to use.")

    path_to_file = Path(args.file) if args.file else None

    module = import_from_file(path_to_file) if path_to_file else None

    scene_class: type[Scene] = getattr(module, args.scene) if module else Scene
    plugins = args.plugins or []
    plugins: list[ModuleType] = [
        import_from_file(Path(plugin))
        for plugin in plugins
    ]
    config.write_to_movie = False

    ManimStudioAPI.enabled = True
    scene = scene_class()
    path_to_file = str(path_to_file) if path_to_file else None
    ManimStudioAPI(scene, module, path_to_file, plugins)
    thread = Thread(target=scene.render, daemon=True)
    thread.start()

    app = QApplication([])
    screen = app.primaryScreen()
    window_size = screen.size()
    window_size_as_tuple = (window_size.width(), window_size.height())
    window = Window(
        window_size_as_tuple,
        ManimStudioAPI.scene
    )
    window.showMaximized()
    app.exec()


if __name__ == "__main__":
    main()
